/*
                                +---------------------------------+
                                |                                 |
                                |     ***   Rasterizer   ***      |
                                |                                 |
                                |  Copyright   -tHE SWINe- 2002  |
                                |                                 |
                                |           Raster.cpp            |
                                |                                 |
                                +---------------------------------+
*/

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <math.h>

#include "Font_Eng.h"
#include "Tga.h"
#include "Load_3ds.h"
#include "Matrix.h"
#include "Pyramid.h"
#include "Seg_man.h"

#include "Raster.h"

#define PI 3.14159265358979323846263279502884

struct Poly_Edge {                            
    int x, y, num, dxdy;            
};

static int n_Width, n_Height;
// vyska a sirka obrazovky

static float z_delta;
static Pyramid pyramid;
// klipovaci pyramida

static Poly_Edge left_edge_buff[7];
static Poly_Edge right_edge_buff[7];
static Poly_Edge *p_left_edge;
static Poly_Edge *p_right_edge;
static int n_left_edge_num, n_right_edge_num;
// edge

static float dzdx, dzdy, begz;
static float dudx, dudy, begu;
static float dvdx, dvdy, begv;
// interpolace

static Scanline *s_buffer;
static int const n_st_segs = 20;
// s-buffer

static int W_0, W_1;
static int A_1, A_2, A_3, B_1;
static unsigned char    *_texture;
static unsigned __int32 *_palette;
// texturovani

static Matrix m, im;
// transformacni matice

static char multab[4][4096]; // 4096 = 64^2
// tabulka pro bilinearni interpolaci

#define MIPMAPNO_MIPMAPPING 0
#define ORDINARY_MIPMAPPING 1
#define BIDIRECT_MIPMAPPING 0
// aspon jedno musi byt 1, ostatni 0
// (prepinac stylu mipmappingu)

static void (_fastcall *Interpolate_Segment)(int u1, int du, int v1, int dv,
	unsigned __int32 *video, unsigned __int32 *v2);
// pointer na kreslici funkci
// (pro prepinani biliner filtrovani on/off)

int Init_Engine(int n_New_Width, int n_New_Height)
{
	int i;

    n_Width = n_New_Width;
    n_Height = n_New_Height;
    // nastav velikost obrazu

    z_delta = (float)fabs((float)n_Width / ((float)tan(90 * (PI / 180) / 2) * 2));
    // perspektivni zkresleni pro hel 90

    SetFOV((float)n_Width - 4, (float)n_Height - 4, z_delta, &pyramid);
    // klipovaci pyramida (je o trochu men kvli nepesnostem)

    InitSManager();
	if((s_buffer = (Scanline*) malloc(n_Height * sizeof(Scanline))) == NULL)
		return 0;
	for(i = 0; i < n_Height; i ++) {
		if(!InitScanline(n_st_segs, &s_buffer[i]))
			return 0;
		s_buffer[i].n_seg_used = 0;
	}
	// inicializuje s-buffer

	PreCalc_Multab();
	// bilerp

	Switch_Bilerp(1);

    return 1;
}

void Terminate_Engine()
{
    UninitSManager();
	free((void*)s_buffer);
    // uvolni pam
}

// predpocita tabulku pro bilinearni interpolaci
void PreCalc_Multab()
{
    int u, v, total;
    int i;

    for(u = 0; u < 64; u ++) {
        for(v = 0; v < 64; v ++) {
            i = u + (v << 6);

            multab[0][i] = (((63 - u) * (63 - v)) >> 6);
            multab[1][i] = ((u * (63 - v)) >> 6);
            multab[2][i] = (((63 - u) * v) >> 6);
            multab[3][i] = ((u * v) >> 6);

            total = multab[0][i] + multab[1][i]
                + multab[2][i] + multab[3][i];

            if(total < 63)
                multab[0][i] += 63 - total;
        }
    }
}

// vykresli texturovany bilinearne filtrovany segment o delce =< 16pix.
static void _fastcall Interpolate_Segment_Bilerp(int u1, int du, int v1, int dv,
	unsigned __int32 *video, unsigned __int32 *v2)
{
	int frac, tex;
	do {
		tex = ((u1 >> 16) & A_1) + ((v1 >> B_1) & A_2);
		// adresa texelu

		frac = ((u1 >> 10) & 0x3f) + ((v1 >> 4) & 0xfc0);
		// desetinna cast adresy (6 bitu z V + 6 bitu z U)

		*video = _palette[(_texture[tex] << 6) + multab[0][frac]] +
			     _palette[(_texture[(tex + 1) & A_3] << 6) + multab[1][frac]] +
				 _palette[(_texture[(tex + W_0) & A_3] << 6) + multab[2][frac]] +
				 _palette[(_texture[(tex + W_1) & A_3] << 6) + multab[3][frac]];
		// bilinearni interpolace

		video ++;
		// nova pozice

		u1 += du;
		v1 += dv;
		// interpolace souradnic
	} while(video < v2);
}

// vykresli texturovany segment o delce =< 16pix.
static void _fastcall Interpolate_Segment_Raw(int u1, int du, int v1, int dv,
	unsigned __int32 *video, unsigned __int32 *v2)
{
	do {
		*video = _palette[(_texture[((u1 >> 16) & A_1) + ((v1 >> B_1) & A_2)] << 6) + 63];
		// klasika texturovani

		video ++;
		// nova pozice

		u1 += du;
		v1 += dv;
		// interpolace souradnic
	} while(video < v2);
}

void Switch_Bilerp(int nActivate)
{
	if(nActivate)
		Interpolate_Segment = Interpolate_Segment_Bilerp;
	else
		Interpolate_Segment = Interpolate_Segment_Raw;
	// nastavi pointer na texturovaci funkci
	// pri kresleni se pak nemusi rozhodovat
	// kterou pouzit,staci "zavolat pointer"
}

int Add_Edge(Vertex *v_cur, Vertex *v_next, Poly_Edge *edge)
{
    float y1, y2;
    float dxdy;
    int num;

    y1 = (float)ceil(v_cur->y);
    y2 = (float)ceil(v_next->y);
    num = (int)(y2 - y1);

    if(!num)
        return 0;

    dxdy = (v_next->x - v_cur->x) / (v_next->y - v_cur->y);
    edge->dxdy = (int)(dxdy * 65536);
    edge->x = (int)((v_cur->x + (y1 - v_cur->y) * dxdy) * 65536);
    edge->y = (int)y1;
    edge->num = num;
    // spot prvn x a y, sklon (dxdy) a poet obrazovch dek

    return 1;
}

int Create_Edges(Polygon *p_poly)
{
    int n_bottom, n_top, n_cur;
    float f_min_y;
    float f_max_y;

    n_left_edge_num = 0;
    n_right_edge_num = 0;
    // poty hran

    f_min_y =  0x10000;
    f_max_y = -0x10000;
    n_cur = 0;
    while(n_cur < p_poly->n_vertex_num) {
        if(p_poly->vertex[n_cur].y < f_min_y) {
            f_min_y = p_poly->vertex[n_cur].y;
            n_top = n_cur;
        }
        if(p_poly->vertex[n_cur].y > f_max_y) {
            f_max_y = p_poly->vertex[n_cur].y;
            n_bottom = n_cur;
        }
        n_cur ++;
    }
    // najde nejvrchnj a nejspodnj vrchol

    if((int)ceil(f_min_y) >= (int)ceil(f_max_y))
        return 0;
    // zkontroluje zda se vejde do rastru

    n_cur = n_top;
    while(n_cur != n_bottom) {
        if(Add_Edge(&p_poly->vertex[n_cur], &p_poly->vertex[(n_cur + 1) %
           p_poly->n_vertex_num], &right_edge_buff[n_right_edge_num]))
            n_right_edge_num ++;
        n_cur = (n_cur + 1) % p_poly->n_vertex_num;
    }
    // sjede pravou vtev

    n_cur = n_bottom;
    while(n_cur != n_top) {
        if(Add_Edge(&p_poly->vertex[(n_cur + 1) % p_poly->n_vertex_num],
           &p_poly->vertex[n_cur], &left_edge_buff[n_left_edge_num]))
            n_left_edge_num ++;
        n_cur = (n_cur + 1) % p_poly->n_vertex_num;
    }

    p_left_edge = left_edge_buff + n_left_edge_num - 1;
    p_right_edge = right_edge_buff;

    return 1;
}

// spocita texturovaci koeficienty
int Compute_Interpolation_Consts(Polygon *pp, int nWidth, int nHeight)
{
	float x[3], y[3], z[3];
	float u[3], v[3];
	float c;
	int i;

	for(i = 0; i < 3; i ++) {    
		z[i] = 1 / pp->vertex[i].z; 
		u[i] = pp->vertex[i].u * z[i] * nWidth;
		v[i] = pp->vertex[i].v * z[i] * nHeight;
		x[i] = pp->vertex[i].x;
		y[i] = pp->vertex[i].y;
	}
	
	x[1] = x[1] - x[0];
	x[2] = x[0] - x[2];
	//
	y[2] = y[2] - y[0];
	y[1] = y[0] - y[1];
	//
	u[1] -= u[0];
	u[2] -= u[0];
	//
	v[1] -= v[0];
	v[2] -= v[0];
	//
	z[1] -= z[0];
	z[2] -= z[0];
	// spocita vektory -musi byt jinak indexovane- (..a "x", "y" jsou zaporne)

	c = x[1] * y[2] - y[1] * x[2];
	// cast rovnice roviny ktera je pro
	// vsechny souradnice konstantni

	if(c == 0) 
		return 0;
	     
	x[1] /= c;    
	y[1] /= c;  
	x[2] /= c;   
	y[2] /= c;
	// predpocitane deleni - mala optimalizace
	// dudx = - a / c; to "-" odstrani zaporne vektory pro x a y
	// a to deleni "/c" odstrani tyhle 4 radky.
	//
	// dudx = u1 * (-y2 / c) + u2 * (-y1 / c) = -(u1 * y2 + u2 * y1) / c

	dudx = u[1] * y[2] + u[2] * y[1];
	dudy = u[1] * x[2] + u[2] * x[1];
	begu = u[0] - x[0] * dudx - y[0] * dudy;

	dvdx = v[1] * y[2] + v[2] * y[1];
	dvdy = v[1] * x[2] + v[2] * x[1];
	begv = v[0] - x[0] * dvdx - y[0] * dvdy;

	dzdx = z[1] * y[2] + z[2] * y[1];
	dzdy = z[1] * x[2] + z[2] * x[1];
	begz = z[0] - x[0] * dzdx - y[0] * dzdy;

	return 1;
}

void Draw_Segment(int c, int l, float u,
	float v, float z, unsigned __int32 *video)
{
    float dz, z1, z2;
    int du, u1, u2;
    int dv, v1, v2;

    z1 = (0x10000 / z);
    // pevod zptky na linern, do 16:16 fp

    u1 = (int)(u * z1);
    v1 = (int)(v * z1);
    // u a v se pevede troku jinak

    while(c >= 16) {
        z += dzdx * 16;
        u += dudx * 16;
        v += dvdx * 16;
        // pro optimalizaci pepotme jen kazd 16. pixel

        z2 = (0x10000 / z);
        // uren daliho linernho z
        
        u2 = (int)(u * z2);
        v2 = (int)(v * z2);
        // u a v se pevede troku jinak

        dz = (z2 - z1) / 16;
        du = (u2 - u1) / 16;
        dv = (v2 - v1) / 16;
        // spot delty

        Interpolate_Segment(u1, du, v1, dv, video, video + 16);
        // kreslc smyka

        c -= 16;
        // sn poet pixel o 16
        
        video += 16;
        // posune se dl

        z1 = z2;
        u1 = u2;
        v1 = v2;
        // vyuije pedpotan hodnoty
    }

    if(c > 0) { // kresl zbytek lajny (pokud dlka nen nsobkem 16)
        z += dzdx * c;
        u += dudx * c;
        v += dvdx * c;

        z2 = (0x10000 / z);
        u2 = (int)(u * z2);
        v2 = (int)(v * z2);

        dz = (z2 - z1) / c;
        du = (u2 - u1) / c;
        dv = (v2 - v1) / c;

        Interpolate_Segment(u1, du, v1, dv, video, video + c);
    }
}

// vycisti s-buffer
void Init_Screen()
{
	register int i;

	for(i = 0; i < n_Height; i ++)
		s_buffer[i].n_seg_used = 0;
}

// vykresli obsah s-bufferu
int Release_Screen(unsigned __int32 *vram)
{
	Segment *act;
	register int j;
	unsigned __int32 *tmp;
	int i, broken = 0, k = 0;
	int l, c;

	tmp = vram;
	for(i = 0; i < n_Height; i ++){
		act = s_buffer[i].segm;
		for(j = 0; j < s_buffer[i].n_seg_used; j ++, act = act->next) {
			if(j < s_buffer[i].n_seg_used - 1) {
				if(act->x2 > act->next->x1)
					broken ++;
			}

			l = act->x1;
			c = act->x2;
			if(l < 0)
				l = 0;
			if(c > n_Width)
				c = n_Width - 1;
			c -= l;
			if(c <= 0)
				continue;
			// oklipuje segment
			
			_palette = act->texture->palette;
			_texture = act->texture->mip_map[act->mip_level].texture;
			A_1 = act->texture->mip_map[act->mip_level].A_1;
			A_2 = act->texture->mip_map[act->mip_level].A_2;
			A_3 = act->texture->mip_map[act->mip_level].A_3;
			B_1 = act->texture->mip_map[act->mip_level].B_1;
			//
			W_0 = act->texture->mip_map[act->mip_level].n_Width;
			W_1 = W_0 + 1;
			// nastavi se aktualni textura

			dudx = act->dudx;
			dvdx = act->dvdx;
			dzdx = act->dzdx;
			// nastavi se interpolacni konstanty

			Draw_Segment(c, l, act->u1 + l * dudx, act->v1 + l * dvdx, act->z1 + l * dzdx, tmp + l);
		}
		k += s_buffer[i].n_seg_used;
		tmp += n_Width;
	}

	if(broken > 0)
		String_Out("S-Buffer narusen - prekreslovani !", 50, 20, vram);
	if(GetMem_Status() == Mem_Insufficient)
		String_Out("Nedostatek pameti RAM !", 50, 50, vram);

	return k;
}

// spocita mip_factor pro ordinalni mipmapu
float Calc_Mip_Factor(Polygon *p, Texture *texture)
{
    float scr_area, txt_area;
    float x1, y1, u1, v1;
    float x2, y2, u2, v2;
    float mip_map_factor;
    int i, v_num, tW, tH;

    v_num = p->n_vertex_num;
    tW = texture->n_Width;
    tH = texture->n_Height;
    scr_area = 0;
    txt_area = 0;

    x1 = p->vertex[v_num - 1].x;
    y1 = p->vertex[v_num - 1].y;
    u1 = p->vertex[v_num - 1].u * tW;
    v1 = p->vertex[v_num - 1].v * tH;

    for(i = 0; i < v_num; i ++) {	
        x2 = p->vertex[i].x;
        y2 = p->vertex[i].y;
        u2 = p->vertex[i].u * tW;
        v2 = p->vertex[i].v * tH;
        //
        scr_area += x1 * y2 - y1 * x2;
        txt_area += u1 * v2 - v1 * u2;
        //
        x1 = x2;
        y1 = y2;
        u1 = u2;
        v1 = v2;
    }

    mip_map_factor = (float)fabs(txt_area / scr_area);
	// spocita 1 / plocha texelu

	return mip_map_factor;
}

float Calc_Mip_Factor_X(Polygon *p, Texture *texture)
{
	float x[2], y[2], u[2];
	float a, b, c;

	a = (float)texture->n_Width;

	x[0] = p->vertex[1].x - p->vertex[0].x;
	x[1] = p->vertex[2].x - p->vertex[0].x;

	y[0] = p->vertex[1].y - p->vertex[0].y;
	y[1] = p->vertex[2].y - p->vertex[0].y;

	u[0] = p->vertex[1].u * a - p->vertex[0].u * a;
	u[1] = p->vertex[2].u * a - p->vertex[0].u * a;

	a = y[0] * u[1] - y[1] * u[0];
	b = u[0] * x[1] - u[1] * x[0];
	c = x[0] * y[1] - x[1] * y[0];
	// cross product - normala polygonu

	if(c == 0) 
		return 0;
	
	a /= c;
	b /= c;

	return (float)(a * a + b * b);
	// spocita 1 / plocha texelu v souradnici u
}

float Calc_Mip_Factor_Y(Polygon *p, Texture *texture)
{
	float x[2], y[2], v[2];
	float a, b, c;

	a = (float)texture->n_Height;

	x[0] = p->vertex[1].x - p->vertex[0].x;
	x[1] = p->vertex[2].x - p->vertex[0].x;

	y[0] = p->vertex[1].y - p->vertex[0].y;
	y[1] = p->vertex[2].y - p->vertex[0].y;

	v[0] = p->vertex[1].v * a - p->vertex[0].v * a;
	v[1] = p->vertex[2].v * a - p->vertex[0].v * a;

	a = y[0] * v[1] - y[1] * v[0];
	b = v[0] * x[1] - v[1] * x[0];
	c = x[0] * y[1] - x[1] * y[0];
	// cross product - normala polygonu

	if(c == 0) 
		return 0;
	
	a /= c;
	b /= c;

	return (float)(a * a + b * b);
	// spocita 1 / plocha texelu v souradnici v
}

// prevede mipfactor na potrebne zmenseni textury
int MipFactor_ToScale(float mip_map_factor)
{
    if(mip_map_factor > 262000)
        return 9;
    else if(mip_map_factor > 65500)
        return 8;
    else if(mip_map_factor > 16000)
        return 7;
    else if(mip_map_factor > 4000)
        return 6;
    else if(mip_map_factor > 1000)
        return 5;
    else if(mip_map_factor > 250)
        return 4;
    else if(mip_map_factor > 70)
        return 3;
    else if(mip_map_factor > 20)
        return 2;
    else if(mip_map_factor > 5)
        return 1;

    return 0;
}

int DrawPolygon(Polygon *p, Texture *texture)
{
    int start_y, left_x, right_x;
	int mip_level_x, mip_level_y;
	Segment tmp_seg;
    float z, u, v;
    int l, c;

    if(!Create_Edges(p)) 
        return 0;

	if(ORDINARY_MIPMAPPING) {
		mip_level_x = MipFactor_ToScale(Calc_Mip_Factor(p, texture));
		mip_level_y = mip_level_x;
	} else if(BIDIRECT_MIPMAPPING) {
		mip_level_x = MipFactor_ToScale(Calc_Mip_Factor_X(p, texture));
		mip_level_y = MipFactor_ToScale(Calc_Mip_Factor_Y(p, texture));
	} else /*if(MIPMAPNO_MIPMAPPING)*/ {
		mip_level_x = 0;
		mip_level_y = 0;
	}
	// spocita mip-factor

	if(mip_level_x >= texture->n_mip_num_x)
		mip_level_x = texture->n_mip_num_x - 1;
	if(mip_level_y >= texture->n_mip_num_y)
		mip_level_y = texture->n_mip_num_y - 1;
	// zkontroluje zda textura tento faktor ma

    if(!Compute_Interpolation_Consts(p,
		texture->n_Width >> mip_level_x,
	    texture->n_Height >> mip_level_y)) 
        return 0;
	// muzou se spocitat az potom co je znamy mip_level

    start_y = p_right_edge->y;
    left_x = p_left_edge->x;
    right_x = p_right_edge->x;
    // zkopruje vlastnosti edge

    z = begz + dzdy * start_y;
    u = begu + dudy * start_y;
    v = begv + dvdy * start_y;
    // u, v, z prvnho dku rastru

	tmp_seg.dudx = dudx;
	tmp_seg.dvdx = dvdx;
	tmp_seg.dzdx = dzdx;
	tmp_seg.texture = texture;
	// nastav spolen vlastnosti
	// pro segmenty aktulnho polygonu

	tmp_seg.mip_level = mip_level_x + mip_level_y * texture->n_mip_num_x;
	// upravi mip_level aby indexoval odpovidajici mip_mapu

    while(1) {
        c = left_x >> 16;
        l = right_x >> 16;
        // pevod zptky na int

        if(c > 0) {
			tmp_seg.x1 = l;
			tmp_seg.x2 = c;
			tmp_seg.z1 = z;
			tmp_seg.u1 = u;
			tmp_seg.v1 = v;
			// donastav

			AddSegment(&tmp_seg, &s_buffer[start_y]);
			// pid segment do bufferu
		}

        start_y ++;
        z += dzdy;
        u += dudy;
        v += dvdy;
        // posun na dalsi dek obrazovky

        if(-- p_left_edge->num == 0) {
            if(p_left_edge -- == left_edge_buff)
                return 1;
            // pokud dojde na konec bufferu, je konec polygonu
            left_x = p_left_edge->x;
        } else 
            left_x += p_left_edge->dxdy;

        if(-- p_right_edge->num == 0) {
            p_right_edge ++;
            right_x = p_right_edge->x;
        } else 
            right_x += p_right_edge->dxdy;
    }

    return 1;
}

void DrawObject(Matrix *m_object, Matrix *m_camera,
    Object *p_object, Material *p_material)
{
    Face *cur_face, face;
	int n_material_id;
    float tx, ty, tz;
    float x, y, z;
    Polygon poly;
    Vertex v[3];
    int i, j;

    face.vertex[0] = &v[0];
    face.vertex[1] = &v[1];
    face.vertex[2] = &v[2];
    // nastav vertexy pro nai doasnou face

    Matrix_Inverse(m_camera, &m);
    Matrix_Multiply(&m, m_object, &m);
    Matrix_Inverse(&m, &im);
    // spot transforman matice

    for(i = 0; i < p_object->n_face_num; i ++){
        if((p_object->face[i].normal.c * im[3][2] +
           p_object->face[i].normal.a * im[3][0] +
           p_object->face[i].normal.b * im[3][1] +
           p_object->face[i].normal.d) > 0.1)
            continue;
        // byckface culling

        cur_face = &p_object->face[i];
		n_material_id = cur_face->n_material;
        for(j = 0; j < 3; j ++) {
            x = cur_face->vertex[j]->x;
            y = cur_face->vertex[j]->y;
            z = cur_face->vertex[j]->z;
            //
            tx = x * m[0][0] + y * m[1][0] + 
                 z * m[2][0] +     m[3][0];
            ty = x * m[0][1] + y * m[1][1] + 
                 z * m[2][1] +     m[3][1];
            tz = x * m[0][2] + y * m[1][2] + 
                 z * m[2][2] +     m[3][2];
            //
            face.vertex[j]->x = tx;
            face.vertex[j]->y = ty;
            face.vertex[j]->z = tz;
            //
			x = cur_face->vertex[j]->normal.x;
            y = cur_face->vertex[j]->normal.y;
            z = cur_face->vertex[j]->normal.z;
            //
            tx = x * m[0][0] + y * m[1][0] + z * m[2][0];
            ty = x * m[0][1] + y * m[1][1] + z * m[2][1];
			//
            face.vertex[j]->u = (tx + 1) / 2;
            face.vertex[j]->v = (ty + 1) / 2;
        }

        poly = Clip_Face(&face, &pyramid);
        // proene face pyramidou

        if(poly.n_vertex_num) { // pokud je face vidt, bude to nenulov slo
            for(j = 0; j < poly.n_vertex_num; j ++) {
                if(poly.vertex[j].z < 0.1) {
                    poly.n_vertex_num = 0;
                    break;
                }
                // pokud je z - souadnice moc mal, zahodme polygon

                poly.vertex[j].x = (n_Width / 2) + (poly.vertex[j].x *
                    z_delta) / poly.vertex[j].z;
                poly.vertex[j].y = (n_Height / 2) + (poly.vertex[j].y *
                    z_delta) / poly.vertex[j].z;
                // perspektivn korekce
            }

            DrawPolygon(&poly, p_material[n_material_id].texture);
            // vykresl polygon na obrazovku
        }
    }
}

int Create_MipMap(Mip_Map *mipmap, int nWidth, int nHeight)
{
	if((mipmap->texture = (unsigned char*) malloc(nWidth * nHeight)) == NULL)
		return 0;
	memset(mipmap->texture, 0, nWidth * nHeight);
	// naalokuje pamet pro mipmapu

	mipmap->n_Width = nWidth;
	mipmap->n_Height = nHeight;
	// nastavi velikost

	mipmap->A_1 = nWidth - 1;
	mipmap->A_2 = nWidth * (nHeight - 1);
	mipmap->A_3 = nWidth * nHeight - 1;
	mipmap->B_1 = 16 - (int)(log((double)nWidth) / log(2) + 1e-7);
	// udela par vypoctu
	// A_1 je maximalni x-ova souradnice
	// A_2 je maximalni y-ova souradnice (nasobena sirkou bitmapy)
	// A_3 je maska indexu textury - potreba pro bilinear. interpolaci
	// B_1 je 16 - mocnina dvou sirky textury (Width = 256 <=> B_1 = 8)

	return 1;
}

// nahraje texturu z filename
Texture *Load_Texture(char *filename)
{
	Mip_Map *act_map;
	Texture *temp;
	int r, g, b;
	TGA *tga;
	int x, y;
	int b_x;
	int b_y;
	int i;

	if((tga = Load_Pal_TGA(filename)) == NULL)
		return NULL;
	// nahraje bitmapu

	if((temp = (Texture*) malloc(sizeof(Texture))) == NULL) {
		free((void*)tga->palette);
		free((void*)tga->bytes);
		free((void*)tga);
		// pokud neuspeje, uvolni bitmapu
		return NULL;
	}
	// naalokuje texturu

	temp->n_mip_num_x = (int)(log((double)tga->Width) / log(2) + 1e-7) - 1;
	temp->n_mip_num_y = (int)(log((double)tga->Height) / log(2) + 1e-7) - 1;
	// spocita pocet horizontalnich a vertikalnich mipmap

	temp->n_Width = tga->Width / 2;
	temp->n_Height = tga->Height / 2;
	// velikost 0-te mipmapy (orig. textura)

	if((temp->mip_map = (Mip_Map*) malloc
	   (temp->n_mip_num_x * temp->n_mip_num_y * sizeof(Mip_Map))) == NULL) {
		free((void*)tga->palette);
		free((void*)tga->bytes);
		free((void*)tga);
		// pokud neuspeje, uvolni bitmapu
		return NULL;
	}
	// naalokuje pamet pro mipmapy

	for(y = 1, b_y = 0; y <= temp->n_mip_num_y; b_y += tga->Height >> (y ++)) {
		for(x = 1, b_x = 0; x <= temp->n_mip_num_x; b_x += tga->Width >> (x ++)) {	
			act_map = &temp->mip_map[(x - 1) + (y - 1) * temp->n_mip_num_x];
			// pointer na aktualni mipmapu

			if(!Create_MipMap(act_map, tga->Width >> x, tga->Height >> y))
				return NULL;
			// pokusi se vytvorit rastr pro mipmapu

			for(i = 0; i < tga->Height >> y; i ++) {
				memcpy(&act_map->texture[i * act_map->n_Width],
					&tga->bytes[b_x + (i + b_y) * tga->Width], act_map->n_Width);
			}
			// zkopiruje data z bitmapy
		}
	}
	// vytvari mipmapy

	if((temp->palette = (unsigned __int32*) malloc(64 * tga->color_num * 4)) == NULL) {
		free((void*)tga->palette);
		free((void*)tga->bytes);
		free((void*)tga);
		// pokud neuspeje, uvolni bitmapu
		return NULL;
	}
	for(i = 0; i < tga->color_num; i ++) {
		r = tga->palette[i] & 0xff0000;
		g = tga->palette[i] & 0x00ff00;
		b = tga->palette[i] & 0x0000ff;
		//
		for(x = 0; x < 64; x ++) {
			temp->palette[(i * 64) + x] = ((r * x) / 64) & 0xff0000 |
										  ((g * x) / 64) & 0x00ff00 |
										  ((b * x) / 64) & 0x0000ff;
		}
	}
	// nastavi paletu a predpocita odstiny barev (64)

	free((void*)tga->bytes);
	free((void*)tga);
	// uvolni jiz nepotrebnou bitmapu (krome palety)

	return temp;
}

// nahraje textury podle jmen materialu v bsp-stromu
// tree textury jsou hledany ve slozce dir
int Load_Textures(World *world, char *dir)
{
	char path[256];
	int i;

	if(dir[strlen(dir) - 1] == '\\')
		dir[strlen(dir) - 1] = 0;
	// pokud adresar obsahuje lomitko, umaze se
	// (je to jednodussi nez ho pridat ;-) )

	for(i = 0; i < world->n_material_num; i ++) {
		sprintf(path, "%s\\%s.tga", dir, world->material[i].name);
		// vytvori retezec s adresou k bitmape, reprezentujici texturu

		world->material[i].texture = Load_Texture(path);
		// nahraje texturu

		if(world->material[i].texture == NULL)
			return 0;
		// soubor neexistuje..
	}

	return 1;
}

// vygeneruje histogram potu rzn dlohch segment
// (velice moc hodn neoptimalizovan)
void Generate_Scanline_Graph()
{
	Segment *act;
	register int j;
	int i, k, f;
	int l, c, max_c;
    //
	int *seg_count;
	int *seg_length;
	int seg_length_num;

	seg_length_num = 0;
	//
	seg_count = (int*) malloc(0);
	seg_length = (int*) malloc(0);
	//
	for(i = 0; i < n_Height; i ++){
		act = s_buffer[i].segm;
		for(j = 0; j < s_buffer[i].n_seg_used; j ++, act = act->next) {
			l = act->x1;
			c = act->x2;
			if(l < 0)
				l = 0;
			if(c > n_Width)
				c = n_Width - 1;
			c -= l;
			if(c <= 0)
				continue;
			// oklipuje segment

			f = -1;
			for(k = 0; k < seg_length_num; k ++) {
				if(seg_length[k] == c) {
					f = k;
					break;
				}
			}
			//
			if(f != -1)
				seg_count[f] ++;
			else {
				f = seg_length_num ++;
				//
				seg_count = (int*) realloc(seg_count, seg_length_num * sizeof(int));
				seg_length = (int*) realloc(seg_length, seg_length_num * sizeof(int));
				//
				seg_count[f] = 1;
				seg_length[f] = c;
			}
			// najde segmenty se stejnou delkou a pocita je
		}
	}

	do {
		k = 0;
		for(i = 1; i < seg_length_num; i ++) {
			if(seg_length[i - 1] > seg_length[i]) {
				k = 1;
				//
				j = seg_length[i - 1];
				seg_length[i - 1] = seg_length[i];
				seg_length[i] = j;
				//
				j = seg_count[i - 1];
				seg_count[i - 1] = seg_count[i];
				seg_count[i] = j;
			}
		}
	} while(k != 0);
	// seradi pole podle delky segmentu (bubble sort)

	max_c = 0;
	for(i = 0; i < seg_length_num; i ++) {
		if(seg_count[i] > max_c)
			max_c = seg_count[i];
	}
	// najde max. pocet stejnych segmentu

	FILE *graph;
	char str[256];

	graph = fopen("Graph.txt", "w");

	for(i = 0; i < seg_length_num; i ++) {
		sprintf(str, "%4d (%4d) : ", seg_length[i], seg_count[i]);
		for(j = 0; j < (((float)seg_count[i] / (float)max_c) * 50); j ++)
			strcat(str, "*");
		strcat(str, "\n");
		// vytvori hvezdickovy graf

		fprintf(graph, str);
	}

	fclose(graph);
	free((void*)seg_length);
	free((void*)seg_count);
}